package cache

import (
	"sync"
)

var (
	memCache sync.Map
)

// Load returns the value stored in the map for a key, or nil if no value is present. The ok result indicates whether value was found in the map.
func Load(key interface{}) (value interface{}, ok bool) {
	return memCache.Load(key)
}

// Delete deletes the value for a key.
func Delete(key interface{}) {
	memCache.Delete(key)
}

// LoadAndDelete deletes the value for a key, returning the previous value if any. The loaded result reports whether the key was present.
func LoadAndDelete(key interface{}) (value interface{}, loaded bool) {
	return memCache.LoadAndDelete(key)
}

// LoadOrStore returns the existing value for the key if present. Otherwise, it stores and returns the given value. The loaded result is true if the value was loaded, false if stored.
func LoadOrStore(key, value interface{}) (actual interface{}, loaded bool) {
	return memCache.LoadOrStore(key, value)
}

// Range calls f sequentially for each key and value present in the map. If f returns false, range stops the iteration.
func Range(f func(key, value interface{}) bool) {
	memCache.Range(f)
}

// Store sets the value for a key.
func Store(key, value interface{}) {
	memCache.Store(key, value)
}
